import sys
sys.path[:0] = ["/Users/johnl/dev/python/qgl/qgl"]

import pygame
from pygame.locals import *

import qgl
import qgle
import time
import random
import math

r = random.random

class FramesPerSec(object):
    def __init__(self, parent, clock):
        self.fps = qgl.scene.state.Text("fps", "../test/mono.ttf")
        group = qgl.scene.Group()
        group.add(self.fps)
        group.translate = (-2.6,-2.0,-5)
        group.scale = (0.2,0.2,0.5)
        parent.add(group)
        self.last = 0
        self.clock = clock
        
    def tick(self):
        if time.time() > self.last:
            self.last = time.time() + 1
            self.fps.text = "%0.1f"%(self.clock.get_fps())

class ParticleGroup(object):
    def __init__(self, compiler, parent):
        self.compiler = compiler
        self.parent = parent
        self.group = qgl.scene.Group()
        self.parent.add(self.group)
        self.particles = []
        self.use_material = True

    def tick(self):
        self.group.axis = (0,1,0)
        self.group.angle += r() / 5 + 0.5
        dead = []
        for particle in self.particles:
            particle.tick()
            if not particle.alive:
                dead.append(particle)
        if r() > 0.9:
            self.add_particle(math.sin(self.group.angle))
        self.prune(dead)
    
    def add_particle(self, seed):
        p = Particle(self.compiler, seed, self.use_material)
        self.group.add(p.group)
        self.particles.append(p)
        
    def prune(self, dead):
        for particle in dead:
            self.particles.remove(particle)
            self.group.remove(particle.group)

    def toggle(self):
        self.use_material = not self.use_material
        for particle in self.particles:
            particle.toggle(self.use_material)
        if self.use_material:
            print "Turn on materials"
        else:
            print "Turn off materials"
            
            
class Particle(object):
    def __init__(self, compiler, seed, use_material):
        size = abs(seed) / 5 + 0.2
        posn = seed, seed, seed
        self.colour = qgl.scene.state.Color(self.seededColour(seed))
        self.alive = True
        # Create a material for the sphere so that it is not flat and lifeless
        self.material = qgl.scene.state.Material(diffuse = self.colour.rgba, emissive = (0.2, 0.2, 0.2, 1.0))
        self.sphere = qgl.scene.state.Sphere(size, x_segments=24, y_segments=24)
        self.group = qgl.scene.Group()
        self.group.translate = posn
        if use_material:
            self.group.add(self.colour, self.material, self.sphere)
        else:
            self.group.add(self.colour, self.sphere)
        
        self.group.accept(compiler)
        if r() > 0.1:
            self.group.velocity = (r() * seed * 5, (r() + 0.2) * 10, r() * seed * 5, 0.0)
        else:
            self.group.velocity = (r() * 10 - 5, r() * 10 - 5, r() *2 - 1, 0.0)
        # gravity brings all things down
        self.group.acceleration = (0.0, -3.0, 0.0, 0.0)


    def tick(self):
        c = self.colour.rgba
        c = (c[0], c[1], c[2], c[3] - 0.003)
        self.colour.rgba = c
        self.material.ambient = tuple(map(lambda x: x/2, self.colour.rgba))
        self.material.diffuse = self.colour.rgba
        self.group.scale = tuple(map(lambda x: x + 0.005, self.group.scale))
        if c[3] <= 0:
            self.alive = False
            
    def toggle(self, use_material):
        if use_material:
            if self.material in self.group.leaves:
                self.group.remove(self.material)
        else:
            self.group.add(self.material)

    def seededColour(self, seed):
        seed = abs(seed)
        
        def rgb(s):
            l = map(lambda x: r() / 5, range(3)) + [1.0]
            for c in s:
                i = "rgb".index(c)
                l[i] = r() / 5.0 + 0.8
            return tuple(l)
    
        if seed < 0.1667:
            return rgb("r")
        elif seed < 0.3334:
            return rgb("g")
        elif seed < 0.5001:
            return rgb("b")
        elif seed < 0.6668:
            return rgb("rg")
        elif seed < 0.8335:
            return rgb("rb")
        else:
            return rgb("bg")

def main():
    #setup pygame as normal, making sure to include the OPENGL flag in the init function arguments.
    pygame.init()
    flags =  OPENGL|DOUBLEBUF|HWSURFACE
    pygame.display.set_mode((800,600), flags)
    
    
    #Create two visitors.
    #The compiler visitor is used to change a Node object into a set of OpenGL draw commands. More on nodes later.
    compiler = qgl.render.Compiler()
    #The render visitor is used to execute compiled commands.
    render = qgl.render.Render()
    
    #the root node is the root of the tree structure (also called a scene graph). Branches get added to the root. 
    root_node = qgl.scene.Root()
    
    #every root node must have a viewport branch, which specified which area of the screen to draw to.
    #the PersepctiveViewport renders all its children in a 3d view.
    viewport = qgl.scene.PerspectiveViewport()
    viewport.screen_dimensions = (0,0,800,600)
    
    environment = qgl.scene.Group()
    
    #a group node can translate, rotate and scale its children. it can also contain leaves, which are drawable things.
    group = qgl.scene.Group()
    #because this group will be displayed in 3d, using a PerspectiveViewport, it makes sense to move it into the screen
    #using the group.translate attribute. Any objects drawn at a depth (z) of 0.0 in a perspective viewport will not be show.
    group.translate = (0.0,0.0,-30)
    particles = ParticleGroup(compiler, group)
    
    #a Light leaf will control the lighting of any leaves rendered after it.
    light = qgl.scene.state.Light(position=(0,10,20))
    
    #Now we add the different nodes and leaves into a tree structure using the .add method.
    root_node.add(viewport)
    
    # create a frame-per-sec counter
    clock = pygame.time.Clock()

    viewport.add(environment)
    environment.add(light, group)
    fps = FramesPerSec(viewport, clock)
    
    #Before the structure can be drawn, it needs to be compiled. To do this, we ask the root node to accept the compiler visitor.
    #If any new nodes are added later in the program, they must also accept the compiler visitor before they can be drawn.
    root_node.accept(compiler)
    #the main render loop
    print "l - toggles lights\nm - toggles materials\nesc to quit"
    while True:
        #process pygame events.
        for event in pygame.event.get():
            if event.type is QUIT:
                return
            elif event.type is KEYDOWN:
                if event.key == 108: # L
                    if light in environment.leaves:
                        print "Turn off lighting"
                        environment.remove(light)
                    else:
                        print "Turn on lighting"
                        environment.add(light)
                elif event.key == 109: # M
                    particles.toggle()
                elif event.key == 27 or event.key == 113:
                    return
                else:
                    print event.key
        
        # Update the particles group and frames-per-sec counter
        particles.tick()
        fps.tick()

        #ask the root node to accept the render visitor. This will draw the structure onto the screen.
        #notice that QGL draws everything from the centre, and that the 0,0 point in a QGL screen is the center of the screen.
        root_node.accept(render)
        
        #clock.tick(30)
        qgle.drive(root_node, clock.tick(60) / 1000.0)
        #flip the display
        pygame.display.flip()

main()

