#Copyright (c) 2006 Simon Wittber
#
#Permission is hereby granted, free of charge, to any person
#obtaining a copy of this software and associated documentation files
#(the "Software"), to deal in the Software without restriction,
#including without limitation the rights to use, copy, modify, merge,
#publish, distribute, sublicense, and/or sell copies of the Software,
#and to permit persons to whom the Software is furnished to do so,
#subject to the following conditions:
#
#The above copyright notice and this permission notice shall be
#included in all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
#BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
#ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
#CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#SOFTWARE.

from  math import sqrt
import Numeric

from qgl.scene import Group, Transform
import qgl

    
def bounding_radius_squared(node):
    """
    Calculate bounding radius ** 2 for node (and its children).
    Sets a bounding_radius_squared attribute on each node.
    """
    r = 0.0
    for branch in node.branches:
        cr = bounding_radius_squared(branch)
        if cr > r:
            r = cr
    node.bounding_radius_squared = r
    x = []
    y = []
    z = []
    for leaf in getattr(node, 'leaves', []):
        vertices = getattr(leaf, 'vertices', [])
        x.extend([i[0] for i in vertices])
        y.extend([i[1] for i in vertices])
        z.extend([i[2] for i in vertices])
    if x and y and z:
       maxx = max(((abs(min(x))), abs(max(x))))
       maxy = max((abs(min(y)), abs(max(y))))
       maxz = max((abs(min(z)), abs(max(z))))
       r = ((maxx**2) + (maxy**2) + (maxz**2))
    if r > node.bounding_radius_squared:
        node.bounding_radius_squared = r
    return r
    

def bounding_radius(node):
    """
    Calculate bounding radius for node (and its children).
    Sets a bounding_radius attribute on each node.
    """
    r = 0.0
    for branch in node.branches:
        cr = bounding_radius(branch)
        if cr > r:
            r = cr
    node.bounding_radius = r
    x = []
    y = []
    for leaf in getattr(node, 'leaves', []):
        vertices = getattr(leaf, 'vertices', [])
        x.extend([i[0] for i in vertices])
        y.extend([i[1] for i in vertices])
    if x and y:
       maxx = max(((abs(min(x))), abs(max(x))))
       maxy = max((abs(min(y)), abs(max(y))))
       r = sqrt((maxx**2) + (maxy**2))
    if r > node.bounding_radius:
        node.bounding_radius = r
    return r
    

def collisions(node):
    """
    Returns a tuple of collisions in a nodes children.
    Index 0 is a tuple of children that collide with branch 0 etc.
    """
    bounding_radius_squared(node)
    spheres = Numeric.array([i.translate[:3] + (i.bounding_radius_squared,) for i in node.branches], Numeric.Float)
    branches = []
    for current_index, c in enumerate(spheres):
        branches.append(tuple(Numeric.nonzero(((Numeric.sum(((spheres[:,:3]-c[:3])**2),1) ) - c[2]) <= 0)))
    return tuple(branches)
                
        
def collide(position, radius, node):
    """
    Returns tuple of nodes that intersect sphere specified by position and 
    radius.
    """
    bounding_radius_squared(node)
    spheres = Numeric.array([i.translate[:3] + (i.bounding_radius_squared,) for i in node.branches], Numeric.Float)
    return tuple(Numeric.nonzero(((Numeric.sum(((spheres[:,:3]-position)**2),1) ) - (radius**2)) <= 0))
    

def cull(node, radius):
    """
    Disables all branches in a node which do not fall within the radius 
    argument.
    """
    bounding_radius_squared(node)
    for b in node.branches: b.disable()
    c = node.translate[:3]
    for i in collide((-c[0],-c[1],-c[2]),radius, node):
        node.branches[i].enable()
    
       

def drive(node, T=1.0):
    """
    Traverses a tree, adjusting translations by velocity attributes, 
    and adjusting velocities by acceleration and drag attibutes.
    """
    try:
            x,y,z,a = node.velocity
    except AttributeError:
        pass
    else:
        t = node.translate
        node.translate = t[0] + (x*T), t[1] + (y*T), t[2] + (z*T)
        node.angle += (a*T)
        try:
            x,y,z,a = node.acceleration
        except AttributeError:
            pass
        else:
            v = node.velocity
            node.velocity = v[0] + (x*T), v[1] + (y*T), v[2] + (z*T), v[3] + (a*T)
        try:
            d = node.drag
        except AttributeError:
            pass
        else:
            v = node.velocity
            node.velocity = v[0] * (d**T), v[1] * (d**T), v[2] * (d**T), v[3] * (d**T)
            
            
    for branch in node.branches:
        drive(branch, T)
        
